package opale.matrix;
/*
Copyright (C) 2000  Groupe Opale (http://www.opale.ovh.org)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You can visit the web site http://www.opale.ovh.org to obtain more informations about this program and/or to contact the coders.
*/


 

import opale.tools.Debug;
import opale.mathtools.DVect;

/**
* This class implements a full matrix with real coefficients.
* @since Opale-Matrix 0.1
* @author J.M., O.C.
* @date 03/2001
*/
public final class Matrix  implements  Cloneable
{
private double[][] a;
private int n,m;	//row and column dimension

/** 
* Construct an n X m matrix of zeros. 
* @param int n, row dimension.
* @param int m, colum dimension.
*/

public Matrix (int n, int m) 
	{
	this.m = m;
	this.n = n;
	a = new double[n][m];
	}

/** 
* Constructeur de la classe Matrix
*  partir d'un tableau de doubles
*/

public Matrix(double[][] x)
	{ 
	a = new double [n=x.length][m=x[0].length] ;
	if (Debug.On) Debug.print("n = "+n);
	if (Debug.On) Debug.print("m = "+m);
	
	
	for (int i=0;i<n;i++)
		for (int j=0;j<m;j++)
			this.a[i][j]= x[i][j];
	} 

/**
* Get the number of rows.
* @return int, the row dimension.
*/
public int getRowDim() { return n; }
/**
* Get the number of columns.
* @return int, the column dimension.
*/
public int getColumnDim() { return m; }

/** 
* Get a coefficient of the matrix.
* @param int i, row index.
* @param int j, column index.
* @return double, coeff(i,j)
*/
public double get(int i, int j) 
	{
	if (Debug.On) Debug.assert((i<n) && (j<m));

	return a[i][j];
	}
	
	
/** 
* Set a coefficient of the matrix.
* @param int i, row index.
* @param int j, column index.
* @param double x, the value of coeff(i,j)
*/
public void set(int i, int j, double x) 
	{
	if (Debug.On) Debug.assert((i<n) && (j<m));
	a[i][j] = x;
	}

/** 
* Make a copy of the current matrix.
* @return Matrix, the copy
*/
public Matrix copy () 
	{
	Matrix C = new Matrix(n,m);
	for (int i = 0; i < n; i++) 
		for (int j = 0; j < m; j++) 
			C.a[i][j] = a[i][j];
	return C;
	}
/**
* Add an another matrix A to the current matrix.
* @param Matrix A, an another matrix
* @return Matrix, the result
*/
public Matrix add(Matrix A)
	{
	if (Debug.On) Debug.assert((n == A.n) && (m == A.m));
	for (int i=0;i<n;i++)
		for (int j=0;j<m;j++)
			a[i][j]+=A.a[i][j];
	return this;
	}

/**
* Multiply the current matrix by a vector.
* @param DVect x, a vector
* @param DVect, the result A*x
*/
public DVect mul(final DVect x)
	{
	if (Debug.On) Debug.assert(m == x.size());
	
	DVect y = new DVect(n);
	
	for (int i=0;i<n;i++)
		{
		double s=0.;
		for (int j=0;j<m;j++)
			s+=a[i][j]*x.get(j);
		y.set(i,s);
		}
	return y;

	}
	
	
/**
* Return the matrix transpose.
* @return Matrix.
*/
public Matrix transpose()
	{
	Matrix t = new Matrix (n,m);
       	for (int i = 0; i < n; i++) 
		for (int j = 0;j<m;j++) 
			t.a[i][j]=this.a[j][i];
	return t;				        
	
	}


/**
* Get the internal two-dimensional array.
*  @return double[][],    Pointer to the array of matrix elements.
*/

public double[][] getInternalArray() 
	{
	return a;
	}
	
/**
* Get a copy of array of matrix elements.
*  @return double[][],  the array of matrix elements.
*/

public double[][] getArrayCopy() 
	{
	double[][] tab = new double[n][m];
	for (int i = 0; i < n; i++) 
		for (int j = 0; j < m; j++) 
			tab[i][j] = a[i][j];
	return tab;
	}

/**
* This method applies a function on all the matrix elements.
* @return IyFx f, a function.
* @since Opale-matrix 0.11
*/
public void appFunc(final opale.mathtools.IyFx f) 
	{
	for (int i = 0; i < n; i++) 
		for (int j = 0; j < m; j++) 
			a[i][j] = f.val(a[i][j]);
	}


/**
* Print the matrix.
*/
public String toString()
	{
	int i,j;
	double aff;
	//String s="";
	StringBuffer s=new StringBuffer(n*m); // bad approximation of size !!
	
	for (i = 0; i <n; i++) 
		{
		for ( j = 0;j<m;j++) 
			{
			aff=a[i][j];
//	     		s+=aff;
//			s+=" ";
			s.append(aff+" ");
			}
//		s+="\n";
		s.append("\n");
		}
	return s.toString();
	}
	
	
	
/**
* Creates and return a copy of this object.
* @return Object a copy of this object.
* @exception  OutOfMemoryError if not enough memory.
* @see        java.lang.Cloneable
* @since Opale-Matrix 0.11
*/
public Object clone()
	{
	try
		{
		Matrix copy = (Matrix) super.clone();
		copy.n = n;
		copy.m = m;
		copy.a = getArrayCopy();
		
		return copy;
		}
	catch (CloneNotSupportedException e) // ne devrait jamais arriver
		{
		throw new InternalError();
		}
   	}
	
	
}
